import type { InjectionKey, SetupContext, UnwrapRef, ExtractPropTypes } from 'vue'
import type {
  RuleItem,
  ValidateError,
  ValidateFieldsError,
} from 'async-validator'
import type { ComponentSize } from '../constants'
// import type {
//   FormEmits,
//   FormItemProp,
//   FormItemProps,
//   FormItemValidateState,
//   FormLabelWidthContext,
//   FormProps,
// } from '@element-plus/components/form'

type Arrayable<T> = T | T[]

export interface FormItemRule extends RuleItem {
  trigger?: Arrayable<string>
}
export type FormRules = Partial<Record<string, Arrayable<FormItemRule>>>

export type FormValidationResult = Promise<boolean>
export type FormValidateCallback = (
  isValid: boolean,
  invalidFields?: ValidateFieldsError
) => void
export interface FormValidateFailure {
  errors: ValidateError[] | null
  fields: ValidateFieldsError
}

export type FormContext = any &
  UnwrapRef<any> & {
    emit: SetupContext<any>['emit']

    // expose
    addField: (field: FormItemContext) => void
    removeField: (field: FormItemContext) => void
    resetFields: (props?: Arrayable<any>) => void
    clearValidate: (props?: Arrayable<any>) => void
    validateField: (
      props?: Arrayable<any>,
      callback?: FormValidateCallback
    ) => FormValidationResult
  }

export interface FormItemContext {
  // $el: HTMLDivElement | undefined
  // size: ComponentSize
  // validateState: any
  // isGroup: boolean
  // labelId: string
  // inputIds: string[]
  // addInputId: (id: string) => void
  // removeInputId: (id: string) => void
  validate: (
    trigger: string,
    callback?: FormValidateCallback
  ) => FormValidationResult
  // resetField(): void
  // clearValidate(): void
}

export const formContextKey: InjectionKey<FormContext> =
  Symbol('formContextKey')
export const formItemContextKey: InjectionKey<FormItemContext> =
  Symbol('formItemContextKey')
